@@ -391,6 +391,7 @@ void do_swoutcritsec(char *, char *);
 void do_almostcrit(char *, char *);
 void do_atopsarflags(char *, char *);
 void do_pacctdir(char *, char *);
+void do_perfevents(char *, char *);
 
 static struct {
 	char	*tag;
@@ -438,6 +439,7 @@ static struct {
 	{	"swoutcritsec",		do_swoutcritsec,	0, },
 	{	"almostcrit",		do_almostcrit,		0, },
 	{	"atopsarflags",		do_atopsarflags,	0, },
+	{	"perfevents",		do_perfevents,		0, },
 	{	"pacctdir",		do_pacctdir,		1, },
 };
 
@@ -696,7 +698,7 @@ main(int argc, char *argv[])
 	** effective user-id to real user-id
 	*/
         if (! droprootprivs() )
-		cleanstop(42);
+		mcleanstop(42, "failed to drop root privs\n");
 
 	/*
 	** start the engine now .....
@@ -908,11 +910,11 @@ engine(void)
 
 		do
 		{
-			curtlen   = counttasks();
+			curtlen   = counttasks();	// worst-case value
 			curtpres  = realloc(curtpres,
 					curtlen * sizeof(struct tstat));
 
-			ptrverify(curtpres, "Malloc failed for %d tstats\n",
+			ptrverify(curtpres, "Malloc failed for %lu tstats\n",
 								curtlen);
 
 			memset(curtpres, 0, curtlen * sizeof(struct tstat));
@@ -1164,12 +1166,11 @@ readrc(char *path, int syslevel)
 				if (tagname[0] == '#')
 					continue;
 
-				fprintf(stderr,
+				mcleanstop(1,
 					"%s: syntax error line "
 					"%d (no value specified)\n",
 					path, line);
 
-				cleanstop(1);
 				break;		/* not reached */
 
 			   default:
@@ -1179,12 +1180,10 @@ readrc(char *path, int syslevel)
 				if (tagvalue[0] != '#')
 					break;
 
-				fprintf(stderr,
+				mcleanstop(1,
 					"%s: syntax error line "
 					"%d (no value specified)\n",
 					path, line);
-
-				cleanstop(1);
 			}
 
 			/*
