@@ -67,6 +67,9 @@
 #include "prev/photosyst_24.h"
 #include "prev/photoproc_24.h"
 
+#include "prev/photosyst_25.h"
+#include "prev/photoproc_25.h"
+
 
 ///////////////////////////////////////////////////////////////
 // Conversion functions
@@ -234,6 +237,7 @@ struct sstat_21		sstat_21;
 struct sstat_22		sstat_22;
 struct sstat_23		sstat_23;
 struct sstat_24		sstat_24;
+struct sstat_25		sstat_25;
 struct sstat		sstat;
 
 struct tstat_20		tstat_20;
@@ -241,6 +245,7 @@ struct tstat_21		tstat_21;
 struct tstat_22		tstat_22;
 struct tstat_23		tstat_23;
 struct tstat_24		tstat_24;
+struct tstat_25		tstat_25;
 struct tstat		tstat;
 
 struct convertall {
@@ -419,6 +424,36 @@ struct convertall {
 		{sizeof(struct gpu_24),
 			STROFFSET(&tstat_24.gpu, &tstat_24),	justcopy},
 	},
+
+	{SETVERSION(2,5), // 2.4 --> 2.5
+		 sizeof(struct sstat_25),	&sstat_25,
+		 sizeof(struct tstat_25), 	NULL,
+
+		{sizeof(struct cpustat_25),  	&sstat_25.cpu,	justcopy},
+		{sizeof(struct memstat_25),  	&sstat_25.mem,	justcopy},
+		{sizeof(struct netstat_25),  	&sstat_25.net,	justcopy},
+		{sizeof(struct intfstat_25), 	&sstat_25.intf,	justcopy},
+		{sizeof(struct dskstat_25),  	&sstat_25.dsk,	justcopy},
+		{sizeof(struct nfsstat_25),  	&sstat_25.nfs,	justcopy},
+		{sizeof(struct contstat_25), 	&sstat_25.cfs,	justcopy},
+		{sizeof(struct wwwstat_25),  	&sstat_25.www,	justcopy},
+		{sizeof(struct pressure_25),  	&sstat_25.psi,	justcopy},
+		{sizeof(struct gpustat_25),  	&sstat_25.gpu,	justcopy},
+		{sizeof(struct ifbstat_25),  	&sstat_25.ifb,	justcopy},
+
+		{sizeof(struct gen_25),
+			STROFFSET(&tstat_25.gen, &tstat_25),	justcopy},
+		{sizeof(struct cpu_25),
+			STROFFSET(&tstat_25.cpu, &tstat_25),	justcopy},
+		{sizeof(struct dsk_25),
+			STROFFSET(&tstat_25.dsk, &tstat_25),	justcopy},
+		{sizeof(struct mem_25),
+			STROFFSET(&tstat_25.mem, &tstat_25),	justcopy},
+		{sizeof(struct net_25),
+			STROFFSET(&tstat_25.net, &tstat_25),	justcopy},
+		{sizeof(struct gpu_25),
+			STROFFSET(&tstat_25.gpu, &tstat_25),	justcopy},
+	},
 };
 
 int	numconvs = sizeof convs / sizeof(struct convertall);
@@ -500,6 +535,7 @@ static void	writeout(int, void *, int);
 static void	writesamp(int, struct rawrecord *, void *, int, void *,
 				int, int);
 
+static void	copy_file(int, int);
 static void	convert_samples(int, int, struct rawheader *, int,  int);
 static void	do_sconvert(struct sconvstruct *, struct sconvstruct *);
 static void	do_tconvert(void *, void *,
@@ -521,7 +557,7 @@ main(int argc, char *argv[])
 	// verify the command line arguments:
 	// 	optional flags and mandatory input and output filename
 	//
-	if (argc < 3)
+	if (argc < 2)
 		prusage(argv[0]);
 
 	while ((c = getopt(argc, argv, "?t:")) != EOF)
@@ -567,18 +603,10 @@ main(int argc, char *argv[])
 		}
 	}
 
-	if (optind < argc-2)
+	if (optind >= argc)
 		prusage(argv[0]);
 
 	infile  = argv[optind++];
-	outfile = argv[optind++];
-
-	if (strcmp(infile, outfile) == 0)
-	{
-		fprintf(stderr,
-	 	    "input file and output file should not be identical!\n");
-		exit(12);
-	}
 
 	// determine target version (default: latest version)
 	//
@@ -633,11 +661,10 @@ main(int argc, char *argv[])
 		exit(11);
 	}
 
-	if (versionix >= targetix)
+	if (versionix > targetix)
 	{
-		fprintf(stderr,
-			"This version is already up-to-date!\n");
-		exit(0);
+		fprintf(stderr, "Downgrading of version is not supported!\n");
+		exit(11);
 	}
 
 	if (irh.sstatlen != convs[versionix].sstatlen ||
@@ -653,6 +680,20 @@ main(int argc, char *argv[])
  		exit(11);
 	}
 
+	// handle the output file 
+	//
+	if (optind >= argc)
+		exit(0);
+
+	outfile = argv[optind++];
+
+	if (strcmp(infile, outfile) == 0)
+	{
+		fprintf(stderr,
+	 	    "input file and output file should not be identical!\n");
+		exit(12);
+	}
+
 	// open the output file 
 	//
 	if ( (ofd = openout(outfile)) == -1)
@@ -674,9 +715,16 @@ main(int argc, char *argv[])
 	printf("Version of %s: %d.%d\n", outfile,
 			(orh.aversion >> 8) & 0x7f, orh.aversion & 0xff);
 
-	// copy and convert every sample
+	// copy and convert every sample, unless the version of the
+	// input file is identical to the target version (then just copy)
 	//
-	convert_samples(ifd, ofd, &irh, versionix, targetix);
+	if (versionix < targetix)
+		convert_samples(ifd, ofd, &irh, versionix, targetix);
+	else
+		copy_file(ifd, ofd);
+
+	close(ifd);
+	close(ofd);
 
 	return 0;
 }
@@ -824,6 +872,48 @@ do_tconvert(void *curtstat,          void *nexttstat,
 	}
 }
 
+
+//
+// Function to copy input file transparently to output file
+//
+static void
+copy_file(int ifd, int ofd)
+{
+	unsigned char	buf[64*1024];
+	int		nr, nw;
+
+	(void) lseek(ifd, 0, SEEK_SET);
+	(void) lseek(ofd, 0, SEEK_SET);
+
+	while ( (nr = read(ifd, buf, sizeof buf)) > 0)
+	{
+		if ( (nw = write(ofd, buf, nr)) < nr)
+		{
+			if (nw == -1)
+			{
+				perror("write output file");
+				exit(42);
+			}
+			else
+			{
+ 				fprintf(stderr,
+ 					"Output file saturated\n");
+				exit(42);
+			}
+		}
+	}
+
+	if (nr == -1)
+	{
+		perror("read input file");
+		exit(42);
+	}
+	else
+	{
+		printf("Raw file copied (version already up-to-date)\n");
+	}
+}
+
 //
 // Function that opens an existing raw file and
 // verifies the magic number
@@ -911,9 +1001,9 @@ void
 prusage(char *name)
 {
 	fprintf(stderr,
-		"Usage: %s [-t version] rawinput rawoutput\n", name);
+		"Usage: %s [-t version] rawinput [rawoutput]\n", name);
 	fprintf(stderr,
-		"\t\t-t version      target version (default: %d.%d) for output\n",
+		"\t-t version      target version (default: %d.%d) for output\n",
 			(convs[numconvs-1].version >> 8) & 0x7f,
 			 convs[numconvs-1].version  & 0x7f);
 
