@@ -318,7 +318,10 @@ atopsar(int argc, char *argv[])
 				prinow    = i;
 				daylim    = 0;
 				begintime = saved_begintime;
-				rawread();
+
+				if (!rawread())	// reading from named pipe
+					break;	// can only be done once
+
 				printf("\n");
 			}
 		}
@@ -384,7 +387,7 @@ atopsar(int argc, char *argv[])
 	** effective user-id to real user-id
 	*/
 	if (! droprootprivs() )
-		cleanstop(42);
+		mcleanstop(42, "failed to drop root privs\n");
 
 	/*
 	** start live reporting
