@@ -200,7 +200,7 @@ photoproc(struct tstat *tasklist, int maxtask)
 		}
 
 		if (! droprootprivs())
-			cleanstop(42);
+			mcleanstop(42, "failed to drop root privs\n");
 
 		/*
  		** find epoch time of boot moment
@@ -219,22 +219,16 @@ photoproc(struct tstat *tasklist, int maxtask)
 	netatop_probe();
 
 	if (! droprootprivs())
-		cleanstop(42);
+		mcleanstop(42, "failed to drop root privs\n");
 
 	/*
 	** read all subdirectory-names below the /proc directory
 	*/
 	if ( getcwd(origdir, sizeof origdir) == NULL)
-	{
-		perror("save current dir");
-		cleanstop(53);
-	}
+		mcleanstop(53, "failed to save current dir\n");
 
 	if ( chdir("/proc") == -1)
-	{
-		perror("change to /proc");
-		cleanstop(54);
-	}
+		mcleanstop(54, "failed to change to /proc\n");
 
 	dirp = opendir(".");
 
@@ -311,6 +305,17 @@ photoproc(struct tstat *tasklist, int maxtask)
 			{
 				dirtask = opendir(".");
 	
+				/*
+				** due to race condition, opendir() might
+				** have failed (leave task and process-level
+				** directories)
+				*/
+				if( dirtask == NULL )		
+				{
+					if(chdir("../..") == -1); 
+					continue;
+				}
+
 				while ((tent=readdir(dirtask)) && tval<maxtask)
 				{
 					struct tstat *curthr = tasklist+tval;
@@ -341,7 +346,7 @@ photoproc(struct tstat *tasklist, int maxtask)
 					}
 
 					strcpy(curthr->gen.container,
-							curtask->gen.container);
+						curtask->gen.container);
 
 					switch (curthr->gen.state)
 					{
@@ -360,7 +365,7 @@ photoproc(struct tstat *tasklist, int maxtask)
 
 					// read network stats from netatop
 					netatop_gettask(curthr->gen.pid, 't',
-									curthr);
+								curthr);
 
 					// all stats read now
 					tval++;	    /* increment thread-level */
@@ -378,10 +383,7 @@ photoproc(struct tstat *tasklist, int maxtask)
 	closedir(dirp);
 
 	if ( chdir(origdir) == -1)
-	{
-		perror(origdir);
-		cleanstop(55);
-	}
+		mcleanstop(55, "cannot change to %s\n", origdir);
 
 	if (dockstat)
 		supportflags |= DOCKSTAT;
@@ -413,25 +415,25 @@ counttasks(void)
 		if ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
 		{
 			if ( sscanf(linebuf, "%*f %*f %*f %*d/%lu", &nr) < 1)
-				cleanstop(53);
+				mcleanstop(53, "wrong /proc/loadavg\n");
 		}
 		else
-			cleanstop(53);
+			mcleanstop(53, "unreadable /proc/loadavg\n");
 
 		fclose(fp);
 	}
 	else
-		cleanstop(53);
+		mcleanstop(53, "can not open /proc/loadavg\n");
 
 
 	/*
 	** add total number of processes 
 	*/
 	if ( getcwd(origdir, sizeof origdir) == NULL)
-		cleanstop(53);
+		mcleanstop(53, "cannot determine cwd\n");
 
 	if ( chdir("/proc") == -1)
-		cleanstop(53);
+		mcleanstop(53, "cannot change to /proc\n");
 
 	dirp = opendir(".");
 
@@ -447,7 +449,7 @@ counttasks(void)
 	closedir(dirp);
 
 	if ( chdir(origdir) == -1)
-		cleanstop(53);
+		mcleanstop(53, "cannot change to %s\n", origdir);
 
 	return nr;
 }
@@ -717,7 +719,7 @@ procio(struct tstat *curtask)
 		}
 
 		if (! droprootprivs())
-			cleanstop(42);
+			mcleanstop(42, "failed to drop root privs\n");
 	}
 
 	return 1;
@@ -763,19 +765,32 @@ proccmd(struct tstat *curtask)
 
 
 /*
-** store the Docker container ID, retrieved from the cpuset
-** that could look like this:
+** store the Docker container ID, retrieved from the 'cpuset'
+** that might look like this:
 **    /system.slice/docker-af78216c2a230f1aa5dce56cbf[SNAP].scope (e.g. CentOS)
 **    /docker/af78216c2a230f1aa5dce56cbf[SNAP]   (e.g. openSUSE and Ubuntu))
+**
+** docker created by k8s might look like this:
+**    /kubepods/burstable/pod07dbb922-[SNAP]/223dc5e15b[SNAP]
+**
+** In general:
+** - search for last '/' (basename)
+** - check if '/' followed by 'docker-': then skip 'docker-'
+** - take 12 positions for the container ID
+**
+** Return value:
+**	0 - no container
+**	1 - container
 */
-#define	CIDPREFIX	"docker"
 #define	CIDSIZE		12
+#define	SHA256SIZE	64
+#define	DOCKPREFIX	"docker-"
 
 static int
 proccont(struct tstat *curtask)
 {
-	FILE		*fp;
-	char		line[80];
+	FILE	*fp;
+	char	line[256];
 
 	if ( (fp = fopen("cpuset", "r")) != NULL)
 	{
@@ -783,18 +798,32 @@ proccont(struct tstat *curtask)
 
 		if ( fgets(line, sizeof line, fp) )
 		{
-			// default string (so if not used) is "/"
-			if (line[1] && (p = strstr(line, CIDPREFIX)) )
-			{
-				memcpy(curtask->gen.container,
-					p + sizeof CIDPREFIX, CIDSIZE);
+			fclose(fp);
 
-				fclose(fp);
+			// fast check for processes not using cpuset
+			// i.e. anyhow not container
+			if (memcmp(line, "/\n", 3) == 0)
+				return 0;
+
+			// possibly container: find basename in path and
+			// verify that its minimum length is the size of SHA256
+			if ( (p = strrchr(line, '/')) != NULL &&
+			                    strlen(p) >= SHA256SIZE)
+			{
+				p++;
+
+				if (memcmp(p, DOCKPREFIX,
+						sizeof(DOCKPREFIX)-1) == 0)
+					p += sizeof(DOCKPREFIX)-1;
+
+				memcpy(curtask->gen.container, p, CIDSIZE);
 				return 1;
 			}
 		}
-
-		fclose(fp);
+		else
+		{
+			fclose(fp);
+		}
 	}
 
 	return 0;
@@ -803,6 +832,9 @@ proccont(struct tstat *curtask)
 
 /*
 ** open file "smaps" and obtain required info
+** since Linux-4.14, kernel supports "smaps_rollup" which has better
+** performence. check "smaps_rollup" in first call
+** if kernel supports "smaps_rollup", use "smaps_rollup" instead
 */
 static void
 procsmaps(struct tstat *curtask)
@@ -810,14 +842,26 @@ procsmaps(struct tstat *curtask)
 	FILE	*fp;
 	char	line[4096];
 	count_t	pssval;
+	static int procsmaps_firstcall = 1;
+	static char *smapsfile = "smaps";
 
+	if (procsmaps_firstcall)
+	{
+		regainrootprivs();
+		if ( (fp = fopen("/proc/1/smaps_rollup", "r")) )
+		{
+			smapsfile = "smaps_rollup";
+			fclose(fp);
+		}
 
+		procsmaps_firstcall = 0;
+	}
 	/*
  	** open the file (always succeeds, even if no root privs)
 	*/
 	regainrootprivs();
 
-	if ( (fp = fopen("smaps", "r")) )
+	if ( (fp = fopen(smapsfile, "r")) )
 	{
 		curtask->mem.pmem = 0;
 
@@ -845,5 +889,5 @@ procsmaps(struct tstat *curtask)
 	}
 
 	if (! droprootprivs())
-		cleanstop(42);
+		mcleanstop(42, "failed to drop root privs\n");
 }
