@@ -44,6 +44,7 @@
 #include <sys/time.h>
 #include <sys/resource.h>
 #include <unistd.h>
+#include <sys/uio.h>
 
 #include "atop.h"
 #include "showgeneric.h"
@@ -118,6 +119,7 @@ static int	getrawrec  (int, struct rawrecord *, int);
 static int	getrawsstat(int, struct sstat *, int);
 static int	getrawtstat(int, struct tstat *, int, int);
 static int	rawwopen(void);
+static int	readchunk(int, void *, int);
 static int	lookslikedatetome(char *);
 static void	testcompval(int, char *);
 static void	try_other_version(int, int);
@@ -140,6 +142,7 @@ rawwrite(time_t curtime, int numsecs,
 	unsigned long		scomplen = sizeof scompbuf;
 	unsigned long		pcomplen = sizeof(struct tstat) *
 						devtstat->ntaskall;
+	struct iovec 		iov[3];
 
 	/*
 	** first call:
@@ -214,37 +217,30 @@ rawwrite(time_t curtime, int numsecs,
 	if (supportflags & GPUSTAT)
 		rr.flags |= RRGPUSTAT;
 
-	if ( write(rawfd, &rr, sizeof rr) == -1)
-	{
-		fprintf(stderr, "%s - ", rawname);
-		perror("write raw record");
-		if ( ftruncate(rawfd, filestat.st_size) == -1)
-			cleanstop(8);
-		cleanstop(7);
-	}
-
 	/*
-	** write compressed system status structure to file
+	** use 1-writev to make record operation atomic
+	** to avoid write uncompleted record data.
 	*/
-	if ( write(rawfd, scompbuf, scomplen) == -1)
-	{
-		fprintf(stderr, "%s - ", rawname);
-		perror("write raw status record");
-		if ( ftruncate(rawfd, filestat.st_size) == -1)
-			cleanstop(8);
-		cleanstop(7);
-	}
+	iov[0].iov_base = &rr;
+	iov[0].iov_len  = sizeof (rr);
 
-	/*
-	** write compressed list of process status structures to file
-	*/
-	if ( write(rawfd, pcompbuf, pcomplen) == -1)
+	iov[1].iov_base = scompbuf;
+	iov[1].iov_len  = scomplen;
+
+	iov[2].iov_base = pcompbuf;
+	iov[2].iov_len  = pcomplen;
+
+	if ( writev(rawfd, iov, 3) == -1)
 	{
 		fprintf(stderr, "%s - ", rawname);
-		perror("write raw process record");
 		if ( ftruncate(rawfd, filestat.st_size) == -1)
-			cleanstop(8);
-		cleanstop(7);
+			mcleanstop(8,
+			   "failed to write raw/status/process record to %s\n",
+			   rawname);
+
+		mcleanstop(7,
+		   "failed to write raw/status/process record to %s\n",
+		   rawname);
 	}
 
 	free(pcompbuf);
@@ -281,20 +277,12 @@ rawwopen()
 		** read and verify header record
 		*/
 		if ( read(fd, &rh, sizeof rh) < sizeof rh)
-		{
-			fprintf(stderr, "%s - cannot read header\n", rawname);
-			cleanstop(7);
-		}
+			mcleanstop(7, "%s - cannot read header\n", rawname);
 
 		if (rh.magic != MYMAGIC)
-		{
-			fprintf(stderr,
-				"file %s exists but does not contain raw "
+			mcleanstop(7, "file %s exists but does not contain raw "
 				"atop output (wrong magic number)\n", rawname);
 
-			cleanstop(7);
-		}
-
 		if ( rh.sstatlen	!= sizeof(struct sstat)		||
 		     rh.tstatlen	!= sizeof(struct tstat)		||
 	    	     rh.rawheadlen	!= sizeof(struct rawheader)	||
@@ -366,10 +354,10 @@ rawwopen()
 */
 #define	OFFCHUNK	256
 
-void
+int
 rawread(void)
 {
-	int			i, j, rawfd, len;
+	int			i, j, rawfd, len, isregular = 1;
 	char			*py;
 	struct rawheader	rh;
 	struct rawrecord	rr;
@@ -382,7 +370,7 @@ rawread(void)
 	** variables to maintain the offsets of the raw records
 	** to be able to see previous samples again
 	*/
-	off_t			*offlist;
+	off_t			*offlist = NULL;
 	unsigned int		offsize = 0;
 	unsigned int		offcur  = 0;
 	char			lastcmd = 'X', flags;
@@ -463,6 +451,25 @@ rawread(void)
 		free(py);
 	}
 
+	/*
+	** make sure the file is a regular file (seekable) or
+	** a pipe (not seekable)
+	*/
+	if (stat(rawname, &filestat) == -1)
+	{
+		fprintf(stderr, "%s - ", rawname);
+		perror("stat raw file");
+		cleanstop(7);
+	}
+
+	if (!S_ISREG(filestat.st_mode) && !S_ISFIFO(filestat.st_mode))
+	{
+		fprintf(stderr, "raw file must be a regular file or pipe\n");
+		cleanstop(7);
+	}
+
+	isregular = S_ISREG(filestat.st_mode);
+
 	/*
 	** open raw file
 	*/
@@ -510,7 +517,7 @@ rawread(void)
 	/*
 	** read the raw header and verify the magic
 	*/
-	if ( read(rawfd, &rh, sizeof rh) < sizeof rh)
+	if ( readchunk(rawfd, &rh, sizeof rh) < sizeof rh)
 	{
 		fprintf(stderr, "can not read raw file header\n");
 		cleanstop(7);
@@ -582,14 +589,17 @@ rawread(void)
 	/*
 	** allocate a list for backtracking of rawrecord-offsets
 	*/
-	offlist = malloc(sizeof(off_t) * OFFCHUNK);
+	if (isregular)
+	{
+		offlist = malloc(sizeof(off_t) * OFFCHUNK);
 
-	ptrverify(offlist, "Malloc failed for backtrack list\n");
+		ptrverify(offlist, "Malloc failed for backtrack list\n");
 
-	offsize = OFFCHUNK;
+		offsize = OFFCHUNK;
 
-	*offlist = lseek(rawfd, 0, SEEK_CUR);
-	offcur   = 1;
+		*offlist = lseek(rawfd, 0, SEEK_CUR);
+		offcur   = 1;
+	}
 
 	/*
 	** read a raw record header until end-of-file
@@ -607,18 +617,21 @@ rawread(void)
 			** store the offset of the raw record in the offset list
 			** in case of offset list overflow, extend the list
 			*/
-			*(offlist+offcur) = lseek(rawfd, 0, SEEK_CUR) -
+			if (isregular)
+			{
+				*(offlist+offcur) = lseek(rawfd, 0, SEEK_CUR) -
 								rh.rawreclen;
 
-			if ( ++offcur >= offsize )
-			{
-				offlist = realloc(offlist,
+				if ( ++offcur >= offsize )
+				{
+					offlist = realloc(offlist,
 				             (offsize+OFFCHUNK)*sizeof(off_t));
 
-				ptrverify(offlist,
+					ptrverify(offlist,
 				        "Realloc failed for backtrack list\n");
 
-				offsize+= OFFCHUNK;
+					offsize+= OFFCHUNK;
+				}
 			}
 	
 			/*
@@ -628,7 +641,26 @@ rawread(void)
 			if ( (begintime && begintime > secsinday) )
 			{
 				lastcmd = 1;
-				lseek(rawfd, rr.scomplen+rr.pcomplen, SEEK_CUR);
+						
+				if (isregular)
+				{
+					lseek(rawfd, rr.scomplen+rr.pcomplen,
+								SEEK_CUR);
+				}
+				else	// named pipe not seekable
+				{
+					char *dummybuf =
+						malloc(rr.scomplen+rr.pcomplen);
+
+					ptrverify(dummybuf,
+				        "Malloc rawlog pipe buffer failed\n");
+
+					readchunk(rawfd, dummybuf,
+						rr.scomplen+rr.pcomplen);
+
+					free(dummybuf);
+				}
+
 				continue;
 			}
 
@@ -636,9 +668,11 @@ rawread(void)
 
 			if ( (endtime && endtime < secsinday) )
 			{
-				free(offlist);
+				if (isregular)
+					free(offlist);
+
 				close(rawfd);
-				return;
+				return isregular;
 			}
 
 			/*
@@ -748,32 +782,44 @@ rawread(void)
 
 			nrgpus = sstat.gpu.nrgpus;
 
-			(void) fstat(rawfd, &filestat);
+			if (isregular)
+			{
+				(void) fstat(rawfd, &filestat);
 
-			if ( filestat.st_size -
-			     lseek(rawfd, (off_t)0, SEEK_CUR) <= rh.rawreclen)
-				flags |= RRLAST;
+				if ( filestat.st_size -
+			     		lseek(rawfd, (off_t)0, SEEK_CUR)
+							<= rh.rawreclen)
+					flags |= RRLAST;
+			}
 
-			lastcmd = (vis.show_samp)(rr.curtime, rr.interval,
+			do
+			{
+				lastcmd = (vis.show_samp)(rr.curtime,
+				     rr.interval,
 			             &devtstat, &sstat,
 			             rr.nexit, rr.noverflow, flags);
+			}
+			while (!isregular &&
+					(lastcmd == MSAMPPREV	||
+					 lastcmd == MSAMPBRANCH	||
+					 lastcmd == MRESET        ));
 
 			free(devtstat.taskall);
 			free(devtstat.procall);
 			free(devtstat.procactive);
-	
+
 			switch (lastcmd)
 			{
-			   case MSAMPPREV:
+		   	   case MSAMPPREV:
 				if (offcur >= 2)
 					offcur-= 2;
 				else
 					offcur = 0;
-	
+
 				lseek(rawfd, *(offlist+offcur), SEEK_SET);
 				break;
 
-			   case MRESET:
+		   	   case MRESET:
 				lseek(rawfd, *offlist, SEEK_SET);
 				offcur = 1;
 				break;
@@ -781,7 +827,8 @@ rawread(void)
 			   case MSAMPBRANCH:
 				if (begintime && begintime < secsinday)
 				{
-					lseek(rawfd, *offlist, SEEK_SET);
+					lseek(rawfd, *offlist,
+							SEEK_SET);
 					offcur = 1;
 				}
 			}
@@ -789,15 +836,25 @@ rawread(void)
 
 		begintime = 0;	// allow earlier times from now on
 
-		if (offcur >= 1)
-			offcur--;
+		if (isregular)
+		{
+			if (offcur >= 1)
+				offcur--;
 
-		lseek(rawfd, *(offlist+offcur), SEEK_SET);
+			lseek(rawfd, *(offlist+offcur), SEEK_SET);
+		}
+		else
+		{
+			lastcmd = 'q';
+		}
 	}
 
-	free(offlist);
+	if (isregular)
+		free(offlist);
 
 	close(rawfd);
+
+	return isregular;
 }
 
 /*
@@ -806,7 +863,7 @@ rawread(void)
 static int
 getrawrec(int rawfd, struct rawrecord *prr, int rrlen)
 {
-	return read(rawfd, prr, rrlen);
+	return readchunk(rawfd, prr, rrlen);
 }
 
 /*
@@ -819,11 +876,11 @@ getrawsstat(int rawfd, struct sstat *sp, int complen)
 	unsigned long	uncomplen = sizeof(struct sstat);
 	int		rv;
 
-	if ( (compbuf = malloc(complen)) == NULL)
+	compbuf = malloc(complen);
 
 	ptrverify(compbuf, "Malloc failed for reading compressed sysstats\n");
 
-	if ( read(rawfd, compbuf, complen) < complen)
+	if ( readchunk(rawfd, compbuf, complen) < complen)
 	{
 		free(compbuf);
 		return 0;
@@ -852,7 +909,7 @@ getrawtstat(int rawfd, struct tstat *pp, int complen, int ndeviat)
 
 	ptrverify(compbuf, "Malloc failed for reading compressed procstats\n");
 
-	if ( read(rawfd, compbuf, complen) < complen)
+	if ( readchunk(rawfd, compbuf, complen) < complen)
 	{
 		free(compbuf);
 		return 0;
@@ -902,27 +959,47 @@ testcompval(int rv, char *func)
 		break;
 
 	   case Z_MEM_ERROR:
-		fprintf(stderr, "atop/atopsar - "
+		mcleanstop(7, "atop/atopsar - "
 		        "%s: failed due to lack of memory\n", func);
-		cleanstop(7);
 
 	   case Z_BUF_ERROR:
-		fprintf(stderr, "atop/atopsar - "
+		mcleanstop(7, "atop/atopsar - "
 			"%s: failed due to lack of room in buffer\n", func);
-		cleanstop(7);
 
    	   case Z_DATA_ERROR:
-		fprintf(stderr, "atop/atopsar - "
+		mcleanstop(7, "atop/atopsar - "
 		        "%s: failed due to corrupted/incomplete data\n", func);
-		cleanstop(7);
 
 	   default:
-		fprintf(stderr, "atop/atopsar - "
+		mcleanstop(7, "atop/atopsar - "
 		        "%s: unexpected error %d\n", func, rv);
-		cleanstop(7);
 	}
 }
 
+static int
+readchunk(int fd, void *buf, int len)
+{
+	char	*p = buf;
+	int	n;
+
+	while (len > 0)
+	{
+ 		switch (n = read(fd, p, len))
+		{
+		   case 0:
+			return 0;	// EOF
+		   case -1:
+			perror("read raw file");
+			cleanstop(9);
+		   default:
+			len -= n;
+			p   += n;
+		}
+	}
+
+	return (char *)p - (char *)buf;
+}
+
 /*
 ** try to activate another atop- or atopsar-version
 ** to read this logfile
